@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (C) 2014-2020 Greenbone Networks GmbH
+# Copyright (C) 2014-2021 Greenbone Networks GmbH
 #
 # SPDX-License-Identifier: AGPL-3.0-or-later
 #
@@ -46,7 +46,7 @@ from ospd_openvas import __version__
 from ospd_openvas.errors import OspdOpenvasError
 
 from ospd_openvas.nvticache import NVTICache
-from ospd_openvas.db import MainDB, BaseDB, ScanDB
+from ospd_openvas.db import MainDB, BaseDB
 from ospd_openvas.lock import LockFile
 from ospd_openvas.preferencehandler import PreferenceHandler
 from ospd_openvas.openvas import Openvas
@@ -359,9 +359,11 @@ OSPD_PARAMS = {
     },
 }
 
+VT_BASE_OID = "1.3.6.1.4.1.25623."
+
 
 def safe_int(value: str) -> Optional[int]:
-    """ Convert a string into an integer and return None in case of errors
+    """Convert a string into an integer and return None in case of errors
     during conversion
     """
     try:
@@ -372,8 +374,7 @@ def safe_int(value: str) -> Optional[int]:
 
 class OpenVasVtsFilter(VtsFilter):
 
-    """ Methods to overwrite the ones in the original class.
-    """
+    """Methods to overwrite the ones in the original class."""
 
     def __init__(self, nvticache: NVTICache) -> None:
         super().__init__()
@@ -381,7 +382,7 @@ class OpenVasVtsFilter(VtsFilter):
         self.nvti = nvticache
 
     def format_vt_modification_time(self, value: str) -> str:
-        """ Convert the string seconds since epoch into a 19 character
+        """Convert the string seconds since epoch into a 19 character
         string representing YearMonthDayHourMinuteSecond,
         e.g. 20190319122532. This always refers to UTC.
         """
@@ -389,7 +390,7 @@ class OpenVasVtsFilter(VtsFilter):
         return datetime.utcfromtimestamp(int(value)).strftime("%Y%m%d%H%M%S")
 
     def get_filtered_vts_list(self, vts, vt_filter: str) -> Optional[List[str]]:
-        """ Gets a collection of vulnerability test from the redis cache,
+        """Gets a collection of vulnerability test from the redis cache,
         which match the filter.
 
         Arguments:
@@ -490,8 +491,7 @@ class OSPDopenvas(OSPDaemon):
         self.initialized = True
 
     def set_params_from_openvas_settings(self):
-        """ Set OSPD_PARAMS with the params taken from the openvas executable.
-        """
+        """Set OSPD_PARAMS with the params taken from the openvas executable."""
         param_list = Openvas.get_settings()
 
         for elem in param_list:
@@ -501,7 +501,7 @@ class OSPDopenvas(OSPDaemon):
                 OSPD_PARAMS[elem]['default'] = param_list[elem]
 
     def feed_is_outdated(self, current_feed: str) -> Optional[bool]:
-        """ Compare the current feed with the one in the disk.
+        """Compare the current feed with the one in the disk.
 
         Return:
             False if there is no new feed.
@@ -546,7 +546,7 @@ class OSPDopenvas(OSPDaemon):
         )
 
     def check_feed(self):
-        """ Check if there is a feed update.
+        """Check if there is a feed update.
 
         Wait until all the running scans finished. Set a flag to announce there
         is a pending feed update, which avoids to start a new scan.
@@ -591,7 +591,7 @@ class OSPDopenvas(OSPDaemon):
 
     @staticmethod
     def get_custom_vt_as_xml_str(vt_id: str, custom: Dict) -> str:
-        """ Return an xml element with custom metadata formatted as string.
+        """Return an xml element with custom metadata formatted as string.
         Arguments:
             vt_id: VT OID. Only used for logging in error case.
             custom: Dictionary with the custom metadata.
@@ -612,7 +612,7 @@ class OSPDopenvas(OSPDaemon):
 
     @staticmethod
     def get_severities_vt_as_xml_str(vt_id: str, severities: Dict) -> str:
-        """ Return an xml element with severities as string.
+        """Return an xml element with severities as string.
         Arguments:
             vt_id: VT OID. Only used for logging in error case.
             severities: Dictionary with the severities.
@@ -623,13 +623,18 @@ class OSPDopenvas(OSPDaemon):
         _severity = SubElement(_severities, 'severity')
         if 'severity_base_vector' in severities:
             try:
-                _severity.text = severities.get('severity_base_vector')
+                _value = SubElement(_severity, 'value')
+                _value.text = severities.get('severity_base_vector')
             except ValueError as e:
                 logger.warning(
                     "Not possible to parse severity tag for vt %s: %s", vt_id, e
                 )
         if 'severity_origin' in severities:
-            _severity.set('origin', severities.get('severity_origin'))
+            _origin = SubElement(_severity, 'origin')
+            _origin.text = severities.get('severity_origin')
+        if 'severity_date' in severities:
+            _date = SubElement(_severity, 'date')
+            _date.text = severities.get('severity_date')
         if 'severity_type' in severities:
             _severity.set('type', severities.get('severity_type'))
 
@@ -637,7 +642,7 @@ class OSPDopenvas(OSPDaemon):
 
     @staticmethod
     def get_params_vt_as_xml_str(vt_id: str, vt_params: Dict) -> str:
-        """ Return an xml element with params formatted as string.
+        """Return an xml element with params formatted as string.
         Arguments:
             vt_id: VT OID. Only used for logging in error case.
             vt_params: Dictionary with the VT parameters.
@@ -672,7 +677,7 @@ class OSPDopenvas(OSPDaemon):
 
     @staticmethod
     def get_refs_vt_as_xml_str(vt_id: str, vt_refs: Dict) -> str:
-        """ Return an xml element with references formatted as string.
+        """Return an xml element with references formatted as string.
         Arguments:
             vt_id: VT OID. Only used for logging in error case.
             vt_refs: Dictionary with the VT references.
@@ -687,11 +692,12 @@ class OSPDopenvas(OSPDaemon):
                     for xref in value.split(', '):
                         try:
                             _type, _id = xref.split(':', 1)
-                        except ValueError:
+                        except ValueError as e:
                             logger.error(
-                                'Not possible to parse xref %s for VT %s',
+                                'Not possible to parse xref "%s" for VT %s: %s',
                                 xref,
                                 vt_id,
+                                e,
                             )
                             continue
                         vt_ref.set('type', _type.lower())
@@ -709,7 +715,7 @@ class OSPDopenvas(OSPDaemon):
     def get_dependencies_vt_as_xml_str(
         vt_id: str, vt_dependencies: List
     ) -> str:
-        """ Return  an xml element with dependencies as string.
+        """Return  an xml element with dependencies as string.
         Arguments:
             vt_id: VT OID. Only used for logging in error case.
             vt_dependencies: List with the VT dependencies.
@@ -719,9 +725,9 @@ class OSPDopenvas(OSPDaemon):
         vt_deps_xml = Element('dependencies')
         for dep in vt_dependencies:
             _vt_dep = Element('dependency')
-            try:
+            if VT_BASE_OID in dep:
                 _vt_dep.set('vt_id', dep)
-            except (ValueError, TypeError):
+            else:
                 logger.error(
                     'Not possible to add dependency %s for VT %s', dep, vt_id
                 )
@@ -734,7 +740,7 @@ class OSPDopenvas(OSPDaemon):
     def get_creation_time_vt_as_xml_str(
         vt_id: str, vt_creation_time: str
     ) -> str:
-        """ Return creation time as string.
+        """Return creation time as string.
         Arguments:
             vt_id: VT OID. Only used for logging in error case.
             vt_creation_time: String with the VT creation time.
@@ -754,7 +760,7 @@ class OSPDopenvas(OSPDaemon):
     def get_modification_time_vt_as_xml_str(
         vt_id: str, vt_modification_time: str
     ) -> str:
-        """ Return modification time as string.
+        """Return modification time as string.
         Arguments:
             vt_id: VT OID. Only used for logging in error case.
             vt_modification_time: String with the VT modification time.
@@ -774,7 +780,7 @@ class OSPDopenvas(OSPDaemon):
 
     @staticmethod
     def get_summary_vt_as_xml_str(vt_id: str, summary: str) -> str:
-        """ Return summary as string.
+        """Return summary as string.
         Arguments:
             vt_id: VT OID. Only used for logging in error case.
             summary: String with a VT summary.
@@ -792,7 +798,7 @@ class OSPDopenvas(OSPDaemon):
 
     @staticmethod
     def get_impact_vt_as_xml_str(vt_id: str, impact) -> str:
-        """ Return impact as string.
+        """Return impact as string.
 
         Arguments:
             vt_id (str): VT OID. Only used for logging in error case.
@@ -811,7 +817,7 @@ class OSPDopenvas(OSPDaemon):
 
     @staticmethod
     def get_affected_vt_as_xml_str(vt_id: str, affected: str) -> str:
-        """ Return affected as string.
+        """Return affected as string.
         Arguments:
             vt_id: VT OID. Only used for logging in error case.
             affected: String which explain what is affected.
@@ -829,7 +835,7 @@ class OSPDopenvas(OSPDaemon):
 
     @staticmethod
     def get_insight_vt_as_xml_str(vt_id: str, insight: str) -> str:
-        """ Return insight as string.
+        """Return insight as string.
         Arguments:
             vt_id: VT OID. Only used for logging in error case.
             insight: String giving an insight of the vulnerability.
@@ -852,7 +858,7 @@ class OSPDopenvas(OSPDaemon):
         solution_type: Optional[str] = None,
         solution_method: Optional[str] = None,
     ) -> str:
-        """ Return solution as string.
+        """Return solution as string.
         Arguments:
             vt_id: VT OID. Only used for logging in error case.
             solution: String giving a possible solution.
@@ -881,7 +887,7 @@ class OSPDopenvas(OSPDaemon):
         qod_type: Optional[str] = None,
         qod: Optional[str] = None,
     ) -> str:
-        """ Return detection as string.
+        """Return detection as string.
         Arguments:
             vt_id: VT OID. Only used for logging in error case.
             detection: String which explain how the vulnerability
@@ -935,8 +941,8 @@ class OSPDopenvas(OSPDaemon):
         return self._sudo_available
 
     def check(self) -> bool:
-        """ Checks that openvas command line tool is found and
-        is executable. """
+        """Checks that openvas command line tool is found and
+        is executable."""
         has_openvas = Openvas.check()
         if not has_openvas:
             logger.error(
@@ -945,49 +951,50 @@ class OSPDopenvas(OSPDaemon):
             )
         return has_openvas
 
-    def update_progress(self, scan_id: str, current_host: str, msg: str):
-        """ Calculate percentage and update the scan status of a host
-        for the progress bar.
-        Arguments:
-            scan_id: Scan ID to identify the current scan process.
-            current_host: Host in the target to be updated.
-            msg: String with launched and total plugins.
-        """
-        try:
-            launched, total = msg.split('/')
-        except ValueError:
-            return
-
-        try:
-            if float(total) == 0:
-                return
-            elif float(total) == ScanProgress.DEAD_HOST:
-                host_prog = ScanProgress.DEAD_HOST
-            else:
-                host_prog = int((float(launched) / float(total)) * 100)
-        except TypeError:
-            return
-
-        self.set_scan_host_progress(
-            scan_id, host=current_host, progress=host_prog
-        )
-
-    def report_openvas_scan_status(
-        self, scan_db: ScanDB, scan_id: str, current_host: str
-    ):
-        """ Get all status entries from redis kb.
+    def report_openvas_scan_status(self, kbdb: BaseDB, scan_id: str):
+        """Get all status entries from redis kb.
 
         Arguments:
+            kbdb: KB context where to get the status from.
             scan_id: Scan ID to identify the current scan.
-            current_host: Host to be updated.
         """
-        res = scan_db.get_scan_status()
-        while res:
-            self.update_progress(scan_id, current_host, res)
-            res = scan_db.get_scan_status()
+        all_status = kbdb.get_scan_status()
+        all_hosts = dict()
+        finished_hosts = list()
+        for res in all_status:
+            try:
+                current_host, launched, total = res.split('/')
+            except ValueError:
+                continue
+
+            try:
+                if float(total) == 0:
+                    continue
+                elif float(total) == ScanProgress.DEAD_HOST:
+                    host_prog = ScanProgress.DEAD_HOST
+                else:
+                    host_prog = int((float(launched) / float(total)) * 100)
+            except TypeError:
+                continue
+
+            all_hosts[current_host] = host_prog
+
+            if (
+                host_prog == ScanProgress.DEAD_HOST
+                or host_prog == ScanProgress.FINISHED
+            ):
+                finished_hosts.append(current_host)
+
+            logger.debug(
+                '%s: Host %s has progress: %d', scan_id, current_host, host_prog
+            )
+
+        self.set_scan_progress_batch(scan_id, host_progress=all_hosts)
+
+        self.sort_host_finished(scan_id, finished_hosts)
 
     def get_severity_score(self, vt_aux: dict) -> Optional[float]:
-        """ Return the severity score for the given oid.
+        """Return the severity score for the given oid.
         Arguments:
             vt_aux: VT element from which to get the severity vector
         Returns:
@@ -1000,45 +1007,58 @@ class OSPDopenvas(OSPDaemon):
 
             if severity_type == "cvss_base_v2" and severity_vector:
                 return CVSS.cvss_base_v2_value(severity_vector)
+            elif severity_type == "cvss_base_v3" and severity_vector:
+                return CVSS.cvss_base_v3_value(severity_vector)
 
         return None
 
-    def report_openvas_results(
-        self, db: BaseDB, scan_id: str, current_host: str
-    ) -> int:
+    def report_openvas_results(self, db: BaseDB, scan_id: str) -> bool:
         """ Get all result entries from redis kb. """
 
         vthelper = VtHelper(self.nvti)
 
         # Result messages come in the next form, with optional uri field
-        # type ||| hostname ||| port ||| OID ||| value [|||uri]
+        # type ||| host ip ||| hostname ||| port ||| OID ||| value [|||uri]
         all_results = db.get_result()
         res_list = ResultList()
         total_dead = 0
-        total_results = len(all_results)
-
         for res in all_results:
             if not res:
                 continue
 
             msg = res.split('|||')
-            roid = msg[3].strip()
+            roid = msg[4].strip()
             rqod = ''
             rname = ''
-            rhostname = msg[1].strip() if msg[1] else ''
-            host_is_dead = "Host dead" in msg[4] or msg[0] == "DEADHOST"
-            host_deny = "Host access denied" in msg[4]
+            current_host = msg[1].strip() if msg[1] else ''
+            rhostname = msg[2].strip() if msg[2] else ''
+            host_is_dead = "Host dead" in msg[5] or msg[0] == "DEADHOST"
+            host_deny = "Host access denied" in msg[5]
+            start_end_msg = msg[0] == "HOST_START" or msg[0] == "HOST_END"
+            host_count = msg[0] == "HOSTS_COUNT"
             vt_aux = None
 
             # URI is optional and msg list length must be checked
             ruri = ''
-            if len(msg) > 5:
-                ruri = msg[5]
+            if len(msg) > 6:
+                ruri = msg[6]
 
-            if roid and not host_is_dead and not host_deny:
+            if (
+                roid
+                and not host_is_dead
+                and not host_deny
+                and not start_end_msg
+                and not host_count
+            ):
                 vt_aux = vthelper.get_single_vt(roid)
 
-            if not vt_aux and not host_is_dead and not host_deny:
+            if (
+                not vt_aux
+                and not host_is_dead
+                and not host_deny
+                and not start_end_msg
+                and not host_count
+            ):
                 logger.warning('Invalid VT oid %s for a result', roid)
 
             if vt_aux:
@@ -1051,51 +1071,52 @@ class OSPDopenvas(OSPDaemon):
                 rname = vt_aux.get('name')
 
             if msg[0] == 'ERRMSG':
-                # Some errors are generated before a host is scanned
-                # use the hostname passed in the message if
-                # no current host is available.
-                if not current_host and rhostname:
-                    current_host = rhostname
-
                 res_list.add_scan_error_to_list(
                     host=current_host,
                     hostname=rhostname,
                     name=rname,
-                    value=msg[4],
-                    port=msg[2],
+                    value=msg[5],
+                    port=msg[3],
                     test_id=roid,
                     uri=ruri,
                 )
 
-            if msg[0] == 'LOG':
+            elif msg[0] == 'HOST_START' or msg[0] == 'HOST_END':
+                res_list.add_scan_log_to_list(
+                    host=current_host,
+                    name=msg[0],
+                    value=msg[5],
+                )
+
+            elif msg[0] == 'LOG':
                 res_list.add_scan_log_to_list(
                     host=current_host,
                     hostname=rhostname,
                     name=rname,
-                    value=msg[4],
-                    port=msg[2],
+                    value=msg[5],
+                    port=msg[3],
                     qod=rqod,
                     test_id=roid,
                     uri=ruri,
                 )
 
-            if msg[0] == 'HOST_DETAIL':
+            elif msg[0] == 'HOST_DETAIL':
                 res_list.add_scan_host_detail_to_list(
                     host=current_host,
                     hostname=rhostname,
                     name=rname,
-                    value=msg[4],
+                    value=msg[5],
                     uri=ruri,
                 )
 
-            if msg[0] == 'ALARM':
+            elif msg[0] == 'ALARM':
                 rseverity = self.get_severity_score(vt_aux)
                 res_list.add_scan_alarm_to_list(
                     host=current_host,
                     hostname=rhostname,
                     name=rname,
-                    value=msg[4],
-                    port=msg[2],
+                    value=msg[5],
+                    port=msg[3],
                     test_id=roid,
                     severity=rseverity,
                     qod=rqod,
@@ -1104,43 +1125,47 @@ class OSPDopenvas(OSPDaemon):
 
             # To process non-scanned dead hosts when
             # test_alive_host_only in openvas is enable
-            if msg[0] == 'DEADHOST':
+            elif msg[0] == 'DEADHOST':
                 try:
-                    total_dead = int(msg[4])
+                    total_dead = int(msg[5])
                 except TypeError:
                     logger.debug('Error processing dead host count')
 
+            # To update total host count
+            if msg[0] == 'HOSTS_COUNT':
+                try:
+                    count_total = int(msg[5])
+                    logger.debug(
+                        '%s: Set total hosts counted by OpenVAS: %d',
+                        scan_id,
+                        count_total,
+                    )
+                    self.set_scan_total_hosts(scan_id, count_total)
+                except TypeError:
+                    logger.debug('Error processing total host count')
+
         # Insert result batch into the scan collection table.
         if len(res_list):
             self.scan_collection.add_result_list(scan_id, res_list)
-
+            logger.debug(
+                '%s: Inserting %d results into scan collection table',
+                scan_id,
+                len(res_list),
+            )
         if total_dead:
+            logger.debug(
+                '%s: Set dead hosts counted by OpenVAS: %d',
+                scan_id,
+                total_dead,
+            )
             self.scan_collection.set_amount_dead_hosts(
                 scan_id, total_dead=total_dead
             )
 
-        return total_results
-
-    def report_openvas_timestamp_scan_host(
-        self, scan_db: ScanDB, scan_id: str, host: str
-    ):
-        """ Get start and end timestamp of a host scan from redis kb. """
-        timestamp = scan_db.get_host_scan_end_time()
-        if timestamp:
-            self.add_scan_log(
-                scan_id, host=host, name='HOST_END', value=timestamp
-            )
-            return
-
-        timestamp = scan_db.get_host_scan_start_time()
-        if timestamp:
-            self.add_scan_log(
-                scan_id, host=host, name='HOST_START', value=timestamp
-            )
-            return
+        return len(res_list) > 0
 
     def is_openvas_process_alive(
-        self, kbdb: BaseDB, ovas_pid: str, openvas_scan_id: str
+        self, kbdb: BaseDB, ovas_pid: str, scan_id: str
     ) -> bool:
         parent_exists = True
         parent = None
@@ -1152,35 +1177,39 @@ class OSPDopenvas(OSPDaemon):
         except TypeError:
             logger.debug(
                 'Scan with ID %s never started and stopped unexpectedly',
-                openvas_scan_id,
+                scan_id,
             )
             parent_exists = False
 
         is_zombie = False
         if parent and parent.status() == psutil.STATUS_ZOMBIE:
+            logger.debug(
+                ' %s: OpenVAS process is a zombie process',
+                scan_id,
+            )
             is_zombie = True
 
         if (not parent_exists or is_zombie) and kbdb:
-            if kbdb and kbdb.scan_is_stopped(openvas_scan_id):
+            if kbdb and kbdb.scan_is_stopped(scan_id):
                 return True
             return False
 
         return True
 
     def stop_scan_cleanup(  # pylint: disable=arguments-differ
-        self, global_scan_id: str
+        self, scan_id: str
     ):
-        """ Set a key in redis to indicate the wrapper is stopped.
+        """Set a key in redis to indicate the wrapper is stopped.
         It is done through redis because it is a new multiprocess
         instance and it is not possible to reach the variables
-        of the grandchild process. Send SIGUSR2 to openvas to stop
-        each running scan."""
+        of the grandchild process.
+        Indirectly sends SIGUSR1 to the running openvas scan process
+        via an invocation of openvas with the --scan-stop option to
+        stop it."""
 
-        openvas_scan_id, kbdb = self.main_db.find_kb_database_by_scan_id(
-            global_scan_id
-        )
+        kbdb = self.main_db.find_kb_database_by_scan_id(scan_id)
         if kbdb:
-            kbdb.stop_scan(openvas_scan_id)
+            kbdb.stop_scan(scan_id)
             ovas_pid = kbdb.get_scan_process_id()
 
             parent = None
@@ -1191,26 +1220,27 @@ class OSPDopenvas(OSPDaemon):
             except TypeError:
                 logger.debug(
                     'Scan with ID %s never started and stopped unexpectedly',
-                    openvas_scan_id,
+                    scan_id,
                 )
 
             if parent:
                 can_stop_scan = Openvas.stop_scan(
-                    openvas_scan_id,
+                    scan_id,
                     not self.is_running_as_root and self.sudo_available,
                 )
                 if not can_stop_scan:
                     logger.debug(
-                        'Not possible to stop scan process: %s.', parent,
+                        'Not possible to stop scan process: %s.',
+                        parent,
                     )
                     return False
 
                 logger.debug('Stopping process: %s', parent)
 
                 while parent:
-                    try:
-                        parent = psutil.Process(int(ovas_pid))
-                    except psutil.NoSuchProcess:
+                    if parent.is_running():
+                        time.sleep(0.1)
+                    else:
                         parent = None
 
             for scan_db in kbdb.get_scan_databases():
@@ -1223,7 +1253,7 @@ class OSPDopenvas(OSPDaemon):
         scan_prefs = PreferenceHandler(
             scan_id, kbdb, self.scan_collection, self.nvti
         )
-        openvas_scan_id = scan_prefs.prepare_openvas_scan_id_for_openvas()
+        kbdb.add_scan_id(scan_id)
         scan_prefs.prepare_target_for_openvas()
 
         if not scan_prefs.prepare_ports_for_openvas():
@@ -1250,17 +1280,22 @@ class OSPDopenvas(OSPDaemon):
         scan_prefs.prepare_scan_params_for_openvas(OSPD_PARAMS)
         scan_prefs.prepare_reverse_lookup_opt_for_openvas()
         scan_prefs.prepare_alive_test_option_for_openvas()
+
+        # VT preferences are stored after all preferences have been processed,
+        # since alive tests preferences have to be able to overwrite default
+        # preferences of ping_host.nasl for the classic method.
+        scan_prefs.prepare_nvt_preferences()
         scan_prefs.prepare_boreas_alive_test()
 
         # Release memory used for scan preferences.
         del scan_prefs
 
-        if do_not_launch:
+        if do_not_launch or kbdb.scan_is_stopped(scan_id):
             self.main_db.release_database(kbdb)
             return
 
         result = Openvas.start_scan(
-            openvas_scan_id,
+            scan_id,
             not self.is_running_as_root and self.sudo_available,
             self._niceness,
         )
@@ -1274,7 +1309,7 @@ class OSPDopenvas(OSPDaemon):
         logger.debug('pid = %s', ovas_pid)
 
         # Wait until the scanner starts and loads all the preferences.
-        while kbdb.get_status(openvas_scan_id) == 'new':
+        while kbdb.get_status(scan_id) == 'new':
             res = result.poll()
             if res and res < 0:
                 self.stop_scan_cleanup(scan_id)
@@ -1287,16 +1322,16 @@ class OSPDopenvas(OSPDaemon):
 
             time.sleep(1)
 
-        no_id_found = False
         got_results = False
         while True:
-            if not kbdb.target_is_finished(
-                scan_id
-            ) and not self.is_openvas_process_alive(
-                kbdb, ovas_pid, openvas_scan_id
-            ):
+            target_is_finished = kbdb.target_is_finished(scan_id)
+            openvas_process_is_alive = self.is_openvas_process_alive(
+                kbdb, ovas_pid, scan_id
+            )
+            if not target_is_finished and not openvas_process_is_alive:
                 logger.error(
-                    'Task %s was unexpectedly stopped or killed.', scan_id,
+                    'Task %s was unexpectedly stopped or killed.',
+                    scan_id,
                 )
                 self.add_scan_error(
                     scan_id,
@@ -1304,7 +1339,12 @@ class OSPDopenvas(OSPDaemon):
                     host='',
                     value='Task was unexpectedly stopped or killed.',
                 )
-                kbdb.stop_scan(openvas_scan_id)
+
+                # check for scanner error messages before leaving.
+                self.report_openvas_results(kbdb, scan_id)
+
+                kbdb.stop_scan(scan_id)
+
                 for scan_db in kbdb.get_scan_databases():
                     self.main_db.release_database(scan_db)
                 self.main_db.release_database(kbdb)
@@ -1320,61 +1360,26 @@ class OSPDopenvas(OSPDaemon):
             got_results = False
 
             # Check if the client stopped the whole scan
-            if kbdb.scan_is_stopped(openvas_scan_id):
+            if kbdb.scan_is_stopped(scan_id):
+                logger.debug('%s: Scan stopped by the client', scan_id)
+
                 # clean main_db, but wait for scanner to finish.
                 while not kbdb.target_is_finished(scan_id):
+                    logger.debug('%s: Waiting the scan to finish', scan_id)
                     time.sleep(1)
                 self.main_db.release_database(kbdb)
                 return
 
-            self.report_openvas_results(kbdb, scan_id, "")
-
-            res_count = 0
-            for scan_db in kbdb.get_scan_databases():
-                id_aux = scan_db.get_scan_id()
-                if not id_aux:
-                    continue
-
-                if id_aux == openvas_scan_id:
-                    no_id_found = False
-                    current_host = scan_db.get_host_ip()
-
-                    res_count += self.report_openvas_results(
-                        scan_db, scan_id, current_host
-                    )
-                    if res_count > 0:
-                        got_results = True
-
-                    self.report_openvas_scan_status(
-                        scan_db, scan_id, current_host
-                    )
-                    self.report_openvas_timestamp_scan_host(
-                        scan_db, scan_id, current_host
-                    )
-
-                    if scan_db.host_is_finished(openvas_scan_id):
-                        self.report_openvas_scan_status(
-                            scan_db, scan_id, current_host
-                        )
-
-                        self.report_openvas_timestamp_scan_host(
-                            scan_db, scan_id, current_host
-                        )
-                        if current_host:
-                            self.sort_host_finished(
-                                scan_id, finished_hosts=current_host
-                            )
-
-                        kbdb.remove_scan_database(scan_db)
-                        self.main_db.release_database(scan_db)
+            got_results = self.report_openvas_results(kbdb, scan_id)
+            self.report_openvas_scan_status(kbdb, scan_id)
 
             # Scan end. No kb in use for this scan id
-            if no_id_found and kbdb.target_is_finished(scan_id):
+            if kbdb.target_is_finished(scan_id):
+                logger.debug('%s: Target is finished', scan_id)
                 break
 
-            no_id_found = True
-
         # Delete keys from KB related to this scan task.
+        logger.debug('%s: End Target. Release main database', scan_id)
         self.main_db.release_database(kbdb)
 
 
