@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (C) 2014-2020 Greenbone Networks GmbH
+# Copyright (C) 2014-2021 Greenbone Networks GmbH
 #
 # SPDX-License-Identifier: AGPL-3.0-or-later
 #
@@ -24,7 +24,6 @@ collection and store the data in a redis KB in the right format to be used by
 OpenVAS. """
 
 import logging
-import uuid
 import binascii
 
 from enum import IntEnum
@@ -48,6 +47,7 @@ OID_SNMP_AUTH = "1.3.6.1.4.1.25623.1.0.105076"
 OID_PING_HOST = "1.3.6.1.4.1.25623.1.0.100315"
 
 BOREAS_ALIVE_TEST = "ALIVE_TEST"
+BOREAS_ALIVE_TEST_PORTS = "ALIVE_TEST_PORTS"
 BOREAS_SETTING_NAME = "test_alive_hosts_only"
 
 
@@ -62,8 +62,29 @@ class AliveTest(IntEnum):
     ALIVE_TEST_TCP_SYN_SERVICE = 16
 
 
+def alive_test_methods_to_bit_field(
+    icmp: bool, tcp_syn: bool, tcp_ack: bool, arp: bool, consider_alive: bool
+) -> int:
+    """Internally a bit field is used as alive test. This function creates
+    such a bit field out of the supplied alive test methods.
+    """
+
+    icmp_enum = AliveTest.ALIVE_TEST_ICMP if icmp else 0
+    tcp_syn_enum = AliveTest.ALIVE_TEST_TCP_SYN_SERVICE if tcp_syn else 0
+    tcp_ack_enum = AliveTest.ALIVE_TEST_TCP_ACK_SERVICE if tcp_ack else 0
+    arp_enum = AliveTest.ALIVE_TEST_ARP if arp else 0
+    consider_alive_enum = (
+        AliveTest.ALIVE_TEST_CONSIDER_ALIVE if consider_alive else 0
+    )
+
+    bit_field = (
+        icmp_enum | tcp_syn_enum | tcp_ack_enum | arp_enum | consider_alive_enum
+    )
+    return bit_field
+
+
 def _from_bool_to_str(value: int) -> str:
-    """ The OpenVAS scanner use yes and no as boolean values, whereas ospd
+    """The OpenVAS scanner use yes and no as boolean values, whereas ospd
     uses 1 and 0."""
     return 'yes' if value == 1 else 'no'
 
@@ -80,20 +101,16 @@ class PreferenceHandler:
         self.kbdb = kbdb
         self.scan_collection = scan_collection
 
-        self._openvas_scan_id = None
-
         self._target_options = None
+        self._nvts_params = None
 
         self.nvti = nvticache
 
-    def prepare_openvas_scan_id_for_openvas(self):
-        """ Create the openvas scan id and store it in the redis kb.
+    def prepare_scan_id_for_openvas(self):
+        """Create the openvas scan id and store it in the redis kb.
         Return the openvas scan_id.
         """
-        self._openvas_scan_id = str(uuid.uuid4())
-        self.kbdb.add_scan_id(self.scan_id, self._openvas_scan_id)
-
-        return self._openvas_scan_id
+        self.kbdb.add_scan_id(self.scan_id)
 
     @property
     def target_options(self) -> Dict:
@@ -106,8 +123,11 @@ class PreferenceHandler:
         )
         return self._target_options
 
-    def _get_vts_in_groups(self, filters: List[str],) -> List[str]:
-        """ Return a list of vts which match with the given filter.
+    def _get_vts_in_groups(
+        self,
+        filters: List[str],
+    ) -> List[str]:
+        """Return a list of vts which match with the given filter.
 
         Arguments:
             filters A list of filters. Each filter has key, operator and
@@ -153,15 +173,19 @@ class PreferenceHandler:
 
     @staticmethod
     def check_param_type(vt_param_value: str, param_type: str) -> Optional[int]:
-        """ Check if the value of a vt parameter matches with
+        """Check if the value of a vt parameter matches with
         the type founded.
         """
-        if param_type in [
-            'entry',
-            'password',
-            'radio',
-            'sshlogin',
-        ] and isinstance(vt_param_value, str):
+        if (
+            param_type
+            in [
+                'entry',
+                'password',
+                'radio',
+                'sshlogin',
+            ]
+            and isinstance(vt_param_value, str)
+        ):
             return None
         elif param_type == 'checkbox' and (
             vt_param_value == '0' or vt_param_value == '1'
@@ -183,7 +207,8 @@ class PreferenceHandler:
         return 1
 
     def _process_vts(
-        self, vts: Dict[str, Dict[str, str]],
+        self,
+        vts: Dict[str, Dict[str, str]],
     ) -> Tuple[List[str], Dict[str, str]]:
         """ Add single VTs and their parameters. """
         vts_list = []
@@ -247,25 +272,18 @@ class PreferenceHandler:
         return vts_list, vts_params
 
     def prepare_plugins_for_openvas(self) -> bool:
-        """ Get the plugin list to be launched from the Scan Collection
-        and prepare the vts preferences. Store the data in the kb.
+        """Get the plugin list and it preferences from the Scan Collection.
+        The plugin list is immediately stored in the kb.
         """
         nvts = self.scan_collection.get_vts(self.scan_id)
         if nvts:
-            nvts_list, nvts_params = self._process_vts(nvts)
+            nvts_list, self._nvts_params = self._process_vts(nvts)
             # Add nvts list
             separ = ';'
             plugin_list = 'plugin_set|||%s' % separ.join(nvts_list)
-            self.kbdb.add_scan_preferences(self._openvas_scan_id, [plugin_list])
+            self.kbdb.add_scan_preferences(self.scan_id, [plugin_list])
 
-            # Add nvts parameters
-            for key, val in nvts_params.items():
-                item = '%s|||%s' % (key, val)
-                self.kbdb.add_scan_preferences(self._openvas_scan_id, [item])
-
-            nvts_params = None
             nvts_list = None
-            item = None
             plugin_list = None
             nvts = None
 
@@ -273,22 +291,49 @@ class PreferenceHandler:
 
         return False
 
+    def prepare_nvt_preferences(self):
+        """Prepare the vts preferences. Store the data in the kb."""
+
+        items_list = []
+        for key, val in self._nvts_params.items():
+            items_list.append('%s|||%s' % (key, val))
+
+        if items_list:
+            self.kbdb.add_scan_preferences(self.scan_id, items_list)
+
     @staticmethod
     def build_alive_test_opt_as_prefs(
         target_options: Dict[str, str]
-    ) -> List[str]:
-        """ Parse the target options dictionary.
+    ) -> Dict[str, str]:
+        """Parse the target options dictionary.
         Arguments:
             target_options: Dictionary with the target options.
 
         Return:
-            A list with the target options related to alive test method
+            A dict with the target options related to alive test method
             in string format to be added to the redis KB.
         """
-        target_opt_prefs_list = []
-        if target_options and target_options.get('alive_test'):
+        target_opt_prefs_list = {}
+        alive_test = None
+
+        if target_options:
+            # Alive test specified as bit field.
+            alive_test = target_options.get('alive_test')
+            # Alive test specified as individual methods.
+            alive_test_methods = target_options.get('alive_test_methods')
+            # alive_test takes precedence over alive_test_methods
+            if alive_test is None and alive_test_methods:
+                alive_test = alive_test_methods_to_bit_field(
+                    icmp=target_options.get('icmp') == '1',
+                    tcp_syn=target_options.get('tcp_syn') == '1',
+                    tcp_ack=target_options.get('tcp_ack') == '1',
+                    arp=target_options.get('arp') == '1',
+                    consider_alive=target_options.get('consider_alive') == '1',
+                )
+
+        if target_options and alive_test:
             try:
-                alive_test = int(target_options.get('alive_test'))
+                alive_test = int(alive_test)
             except ValueError:
                 logger.debug(
                     'Alive test settings not applied. '
@@ -297,6 +342,8 @@ class PreferenceHandler:
                 )
                 return target_opt_prefs_list
 
+            # No alive test or wrong value, uses the default
+            # preferences sent by the client.
             if alive_test < 1 or alive_test > 31:
                 return target_opt_prefs_list
 
@@ -307,12 +354,9 @@ class PreferenceHandler:
                 value = "yes"
             else:
                 value = "no"
-            target_opt_prefs_list.append(
-                OID_PING_HOST
-                + ':1:checkbox:'
-                + 'Do a TCP ping|||'
-                + '{0}'.format(value)
-            )
+            target_opt_prefs_list[
+                OID_PING_HOST + ':1:checkbox:' + 'Do a TCP ping'
+            ] = value
 
             if (
                 alive_test & AliveTest.ALIVE_TEST_TCP_SYN_SERVICE
@@ -321,12 +365,11 @@ class PreferenceHandler:
                 value = "yes"
             else:
                 value = "no"
-            target_opt_prefs_list.append(
+            target_opt_prefs_list[
                 OID_PING_HOST
                 + ':2:checkbox:'
-                + 'TCP ping tries also TCP-SYN ping|||'
-                + '{0}'.format(value)
-            )
+                + 'TCP ping tries also TCP-SYN ping'
+            ] = value
 
             if (alive_test & AliveTest.ALIVE_TEST_TCP_SYN_SERVICE) and not (
                 alive_test & AliveTest.ALIVE_TEST_TCP_ACK_SERVICE
@@ -334,87 +377,103 @@ class PreferenceHandler:
                 value = "yes"
             else:
                 value = "no"
-            target_opt_prefs_list.append(
+            target_opt_prefs_list[
                 OID_PING_HOST
                 + ':7:checkbox:'
-                + 'TCP ping tries only TCP-SYN ping|||'
-                + '{0}'.format(value)
-            )
+                + 'TCP ping tries only TCP-SYN ping'
+            ] = value
 
             if alive_test & AliveTest.ALIVE_TEST_ICMP:
                 value = "yes"
             else:
                 value = "no"
-            target_opt_prefs_list.append(
-                OID_PING_HOST
-                + ':3:checkbox:'
-                + 'Do an ICMP ping|||'
-                + '{0}'.format(value)
-            )
+            target_opt_prefs_list[
+                OID_PING_HOST + ':3:checkbox:' + 'Do an ICMP ping'
+            ] = value
 
             if alive_test & AliveTest.ALIVE_TEST_ARP:
                 value = "yes"
             else:
                 value = "no"
-            target_opt_prefs_list.append(
-                OID_PING_HOST
-                + ':4:checkbox:'
-                + 'Use ARP|||'
-                + '{0}'.format(value)
-            )
+            target_opt_prefs_list[
+                OID_PING_HOST + ':4:checkbox:' + 'Use ARP'
+            ] = value
 
             if alive_test & AliveTest.ALIVE_TEST_CONSIDER_ALIVE:
                 value = "no"
             else:
                 value = "yes"
-            target_opt_prefs_list.append(
+            target_opt_prefs_list[
                 OID_PING_HOST
                 + ':5:checkbox:'
-                + 'Mark unrechable Hosts as dead (not scanning)|||'
-                + '{0}'.format(value)
-            )
+                + 'Mark unrechable Hosts as dead (not scanning)'
+            ] = value
 
             # Also select a method, otherwise Ping Host logs a warning.
             if alive_test == AliveTest.ALIVE_TEST_CONSIDER_ALIVE:
-                target_opt_prefs_list.append(
-                    OID_PING_HOST + ':1:checkbox:' + 'Do a TCP ping|||yes'
-                )
+                target_opt_prefs_list[
+                    OID_PING_HOST + ':1:checkbox:' + 'Do a TCP ping'
+                ] = 'yes'
+
         return target_opt_prefs_list
 
     def prepare_alive_test_option_for_openvas(self):
         """ Set alive test option. Overwrite the scan config settings."""
         settings = Openvas.get_settings()
-        if settings and self.target_options.get('alive_test'):
+        if settings and (
+            self.target_options.get('alive_test')
+            or self.target_options.get('alive_test_methods')
+        ):
             alive_test_opt = self.build_alive_test_opt_as_prefs(
                 self.target_options
             )
-            self.kbdb.add_scan_preferences(
-                self._openvas_scan_id, alive_test_opt
-            )
+            self._nvts_params.update(alive_test_opt)
 
     def prepare_boreas_alive_test(self):
-        """ Set alive_test for Boreas if boreas scanner config
+        """Set alive_test for Boreas if boreas scanner config
         (BOREAS_SETTING_NAME) was set"""
         settings = Openvas.get_settings()
-        alive_test = -1
+        alive_test = None
+        alive_test_ports = None
+        target_options = self.target_options
 
         if settings:
             boreas = settings.get(BOREAS_SETTING_NAME)
             if not boreas:
                 return
-            alive_test_str = self.target_options.get('alive_test')
-            if alive_test_str is not None:
-                try:
-                    alive_test = int(alive_test_str)
-                except ValueError:
-                    logger.debug(
-                        'Alive test preference for Boreas not set. '
-                        'Invalid alive test value %s.',
-                        alive_test_str,
-                    )
-            # ALIVE_TEST_SCAN_CONFIG_DEFAULT if no alive_test provided
-            else:
+        else:
+            return
+
+        if target_options:
+            alive_test_ports = target_options.get('alive_test_ports')
+            # Alive test was specified as bit field.
+            alive_test = target_options.get('alive_test')
+            # Alive test was specified as individual methods.
+            alive_test_methods = target_options.get('alive_test_methods')
+            # <alive_test> takes precedence over <alive_test_methods>
+            if alive_test is None and alive_test_methods:
+                alive_test = alive_test_methods_to_bit_field(
+                    icmp=target_options.get('icmp') == '1',
+                    tcp_syn=target_options.get('tcp_syn') == '1',
+                    tcp_ack=target_options.get('tcp_ack') == '1',
+                    arp=target_options.get('arp') == '1',
+                    consider_alive=target_options.get('consider_alive') == '1',
+                )
+
+        if alive_test is not None:
+            try:
+                alive_test = int(alive_test)
+            except ValueError:
+                logger.debug(
+                    'Alive test preference for Boreas not set. '
+                    'Invalid alive test value %s.',
+                    alive_test,
+                )
+                # Use default alive test as fall back
                 alive_test = AliveTest.ALIVE_TEST_SCAN_CONFIG_DEFAULT
+        # Use default alive test if no valid alive_test was provided
+        else:
+            alive_test = AliveTest.ALIVE_TEST_SCAN_CONFIG_DEFAULT
 
         # If a valid alive_test was set then the bit mask
         # has value between 31 (11111) and 1 (10000)
@@ -422,14 +481,22 @@ class PreferenceHandler:
             pref = "{pref_key}|||{pref_value}".format(
                 pref_key=BOREAS_ALIVE_TEST, pref_value=alive_test
             )
-            self.kbdb.add_scan_preferences(self._openvas_scan_id, [pref])
+            self.kbdb.add_scan_preferences(self.scan_id, [pref])
 
         if alive_test == AliveTest.ALIVE_TEST_SCAN_CONFIG_DEFAULT:
             alive_test = AliveTest.ALIVE_TEST_ICMP
             pref = "{pref_key}|||{pref_value}".format(
                 pref_key=BOREAS_ALIVE_TEST, pref_value=alive_test
             )
-            self.kbdb.add_scan_preferences(self._openvas_scan_id, [pref])
+            self.kbdb.add_scan_preferences(self.scan_id, [pref])
+
+        # Add portlist if present. Validity is checked on Boreas side.
+        if alive_test_ports is not None:
+            pref = "{pref_key}|||{pref_value}".format(
+                pref_key=BOREAS_ALIVE_TEST_PORTS,
+                pref_value=alive_test_ports,
+            )
+            self.kbdb.add_scan_preferences(self.scan_id, [pref])
 
     def prepare_reverse_lookup_opt_for_openvas(self):
         """ Set reverse lookup options in the kb"""
@@ -447,36 +514,36 @@ class PreferenceHandler:
             rev_lookup_unify = _from_bool_to_str(_rev_lookup_unify)
             items.append('reverse_lookup_unify|||%s' % rev_lookup_unify)
 
-            self.kbdb.add_scan_preferences(self._openvas_scan_id, items)
+            self.kbdb.add_scan_preferences(self.scan_id, items)
 
     def prepare_target_for_openvas(self):
-        """ Get the target from the scan collection and set the target
-        in the kb """
+        """Get the target from the scan collection and set the target
+        in the kb"""
 
         target = self.scan_collection.get_host_list(self.scan_id)
         target_aux = 'TARGET|||%s' % target
-        self.kbdb.add_scan_preferences(self._openvas_scan_id, [target_aux])
+        self.kbdb.add_scan_preferences(self.scan_id, [target_aux])
 
     def prepare_ports_for_openvas(self) -> str:
-        """ Get the port list from the scan collection and store the list
-        in the kb. """
+        """Get the port list from the scan collection and store the list
+        in the kb."""
         ports = self.scan_collection.get_ports(self.scan_id)
         port_range = 'port_range|||%s' % ports
-        self.kbdb.add_scan_preferences(self._openvas_scan_id, [port_range])
+        self.kbdb.add_scan_preferences(self.scan_id, [port_range])
 
         return ports
 
     def prepare_host_options_for_openvas(self):
-        """ Get the excluded and finished hosts from the scan collection and
-        stores the list of hosts that must not be scanned in the kb. """
+        """Get the excluded and finished hosts from the scan collection and
+        stores the list of hosts that must not be scanned in the kb."""
         exclude_hosts = self.scan_collection.get_exclude_hosts(self.scan_id)
 
         if exclude_hosts:
             pref_val = "exclude_hosts|||" + exclude_hosts
-            self.kbdb.add_scan_preferences(self._openvas_scan_id, [pref_val])
+            self.kbdb.add_scan_preferences(self.scan_id, [pref_val])
 
     def prepare_scan_params_for_openvas(self, ospd_params: Dict[str, Dict]):
-        """ Get the scan parameters from the scan collection and store them
+        """Get the scan parameters from the scan collection and store them
         in the kb.
         Arguments:
             ospd_params: Dictionary with the OSPD Params.
@@ -504,11 +571,11 @@ class PreferenceHandler:
             prefs_val.append(key + "|||" + val)
 
         if prefs_val:
-            self.kbdb.add_scan_preferences(self._openvas_scan_id, prefs_val)
+            self.kbdb.add_scan_preferences(self.scan_id, prefs_val)
 
     @staticmethod
     def build_credentials_as_prefs(credentials: Dict) -> List[str]:
-        """ Parse the credential dictionary.
+        """Parse the credential dictionary.
         Arguments:
             credentials: Dictionary with the credentials.
 
@@ -624,14 +691,14 @@ class PreferenceHandler:
         return cred_prefs_list
 
     def prepare_credentials_for_openvas(self) -> bool:
-        """ Get the credentials from the scan collection and store them
-        in the kb. """
+        """Get the credentials from the scan collection and store them
+        in the kb."""
         credentials = self.scan_collection.get_credentials(self.scan_id)
         if credentials:
             cred_prefs = self.build_credentials_as_prefs(credentials)
             if cred_prefs:
                 self.kbdb.add_credentials_to_scan_preferences(
-                    self._openvas_scan_id, cred_prefs
+                    self.scan_id, cred_prefs
                 )
 
         if credentials and not cred_prefs:
@@ -640,7 +707,7 @@ class PreferenceHandler:
         return True
 
     def prepare_main_kbindex_for_openvas(self):
-        """ Store main_kbindex as global preference in the
+        """Store main_kbindex as global preference in the
         kb, used by OpenVAS"""
         ov_maindbid = 'ov_maindbid|||%d' % self.kbdb.index
-        self.kbdb.add_scan_preferences(self._openvas_scan_id, [ov_maindbid])
+        self.kbdb.add_scan_preferences(self.scan_id, [ov_maindbid])
