@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (C) 2014-2020 Greenbone Networks GmbH
+# Copyright (C) 2014-2021 Greenbone Networks GmbH
 #
 # SPDX-License-Identifier: AGPL-3.0-or-later
 #
@@ -23,6 +23,7 @@
 
 import io
 import logging
+import psutil
 
 from unittest import TestCase
 from unittest.mock import patch, Mock, MagicMock
@@ -345,7 +346,7 @@ class TestOspdOpenvas(TestCase):
 
         w = DummyDaemon()
         w._sudo_available = None  # pylint: disable=protected-access
-        w.sudo_available  # pylint: disable=pointless-statement
+        w._is_running_as_root = False  # pylint: disable=protected-access
 
         self.assertTrue(w.sudo_available)
 
@@ -385,7 +386,9 @@ class TestOspdOpenvas(TestCase):
         out = (
             '<severities>'
             '<severity type="cvss_base_v2">'
-            'AV:N/AC:L/Au:N/C:N/I:N/A:N'
+            '<value>AV:N/AC:L/Au:N/C:N/I:N/A:N</value>'
+            '<origin>Greenbone</origin>'
+            '<date>1237458156</date>'
             '</severity>'
             '</severities>'
         )
@@ -461,10 +464,26 @@ class TestOspdOpenvas(TestCase):
 
         out = (
             '<dependencies>'
-            '<dependency vt_id="1.2.3.4"/><dependency vt_id="4.3.2.1"/>'
+            '<dependency vt_id="1.3.6.1.4.1.25623.1.2.3.4"/>'
+            '<dependency vt_id="1.3.6.1.4.1.25623.4.3.2.1"/>'
             '</dependencies>'
         )
-        dep = ['1.2.3.4', '4.3.2.1']
+        dep = ['1.3.6.1.4.1.25623.1.2.3.4', '1.3.6.1.4.1.25623.4.3.2.1']
+        res = w.get_dependencies_vt_as_xml_str(
+            '1.3.6.1.4.1.25623.1.0.100061', dep
+        )
+
+        self.assertEqual(res, out)
+
+    def test_get_dependencies_xml_missing_dep(self):
+        w = DummyDaemon()
+
+        out = (
+            '<dependencies>'
+            '<dependency vt_id="1.3.6.1.4.1.25623.1.2.3.4"/>'
+            '</dependencies>'
+        )
+        dep = ['1.3.6.1.4.1.25623.1.2.3.4', 'file_name.nasl']
         res = w.get_dependencies_vt_as_xml_str(
             '1.3.6.1.4.1.25623.1.0.100061', dep
         )
@@ -686,7 +705,9 @@ class TestOspdOpenvas(TestCase):
     @patch('ospd_openvas.daemon.Path.exists')
     @patch('ospd_openvas.daemon.Path.open')
     def test_feed_is_outdated_true(
-        self, mock_path_open: MagicMock, mock_path_exists: MagicMock,
+        self,
+        mock_path_open: MagicMock,
+        mock_path_exists: MagicMock,
     ):
         read_data = 'PLUGIN_SET = "1235";'
 
@@ -709,7 +730,9 @@ class TestOspdOpenvas(TestCase):
     @patch('ospd_openvas.daemon.Path.exists')
     @patch('ospd_openvas.daemon.Path.open')
     def test_feed_is_outdated_false(
-        self, mock_path_open: MagicMock, mock_path_exists: MagicMock,
+        self,
+        mock_path_open: MagicMock,
+        mock_path_exists: MagicMock,
     ):
         mock_path_exists.return_value = True
 
@@ -737,7 +760,7 @@ class TestOspdOpenvas(TestCase):
         self.assertFalse(res)
         w.feed_is_outdated.assert_not_called()
 
-    @patch('ospd_openvas.daemon.ScanDB')
+    @patch('ospd_openvas.daemon.BaseDB')
     @patch('ospd_openvas.daemon.ResultList.add_scan_log_to_list')
     def test_get_openvas_result(self, mock_add_scan_log_to_list, MockDBClass):
         w = DummyDaemon()
@@ -747,15 +770,15 @@ class TestOspdOpenvas(TestCase):
         w.create_scan('123-456', targets, None, [])
 
         results = [
-            "LOG||||||general/Host_Details||||||Host dead",
+            "LOG|||192.168.0.1|||localhost|||general/Host_Details||||||Host dead",
         ]
         MockDBClass.get_result.return_value = results
         mock_add_scan_log_to_list.return_value = None
 
-        w.report_openvas_results(MockDBClass, '123-456', 'localhost')
+        w.report_openvas_results(MockDBClass, '123-456')
         mock_add_scan_log_to_list.assert_called_with(
-            host='localhost',
-            hostname='',
+            host='192.168.0.1',
+            hostname='localhost',
             name='',
             port='general/Host_Details',
             qod='',
@@ -764,7 +787,7 @@ class TestOspdOpenvas(TestCase):
             value='Host dead',
         )
 
-    @patch('ospd_openvas.daemon.ScanDB')
+    @patch('ospd_openvas.daemon.BaseDB')
     @patch('ospd_openvas.daemon.ResultList.add_scan_error_to_list')
     def test_get_openvas_result_host_deny(
         self, mock_add_scan_error_to_list, MockDBClass
@@ -776,15 +799,15 @@ class TestOspdOpenvas(TestCase):
         w.create_scan('123-456', targets, None, [])
 
         results = [
-            "ERRMSG|||127.0.0.1|||||||||Host access denied.",
+            "ERRMSG|||127.0.0.1|||localhost|||||||||Host access denied.",
         ]
         MockDBClass.get_result.return_value = results
         mock_add_scan_error_to_list.return_value = None
 
-        w.report_openvas_results(MockDBClass, '123-456', '')
+        w.report_openvas_results(MockDBClass, '123-456')
         mock_add_scan_error_to_list.assert_called_with(
             host='127.0.0.1',
-            hostname='127.0.0.1',
+            hostname='localhost',
             name='',
             port='',
             test_id='',
@@ -792,7 +815,7 @@ class TestOspdOpenvas(TestCase):
             value='Host access denied.',
         )
 
-    @patch('ospd_openvas.daemon.ScanDB')
+    @patch('ospd_openvas.daemon.BaseDB')
     def test_get_openvas_result_dead_hosts(self, MockDBClass):
         w = DummyDaemon()
         target_element = w.create_xml_target()
@@ -800,17 +823,62 @@ class TestOspdOpenvas(TestCase):
         w.create_scan('123-456', targets, None, [])
 
         results = [
-            "DEADHOST||| ||| ||| |||4",
+            "DEADHOST||| ||| ||| ||| |||4",
         ]
         MockDBClass.get_result.return_value = results
         w.scan_collection.set_amount_dead_hosts = MagicMock()
 
-        w.report_openvas_results(MockDBClass, '123-456', 'localhost')
+        w.report_openvas_results(MockDBClass, '123-456')
         w.scan_collection.set_amount_dead_hosts.assert_called_with(
-            '123-456', total_dead=4,
+            '123-456',
+            total_dead=4,
         )
 
-    @patch('ospd_openvas.daemon.ScanDB')
+    @patch('ospd_openvas.daemon.BaseDB')
+    @patch('ospd_openvas.daemon.ResultList.add_scan_log_to_list')
+    def test_get_openvas_result_host_start(
+        self, mock_add_scan_log_to_list, MockDBClass
+    ):
+        w = DummyDaemon()
+        target_element = w.create_xml_target()
+        targets = OspRequest.process_target_element(target_element)
+        w.create_scan('123-456', targets, None, [])
+
+        results = [
+            "HOST_START|||192.168.10.124||| ||| ||||||today 1",
+        ]
+
+        MockDBClass.get_result.return_value = results
+        mock_add_scan_log_to_list.return_value = None
+
+        w.report_openvas_results(MockDBClass, '123-456')
+
+        mock_add_scan_log_to_list.assert_called_with(
+            host='192.168.10.124',
+            name='HOST_START',
+            value='today 1',
+        )
+
+    @patch('ospd_openvas.daemon.BaseDB')
+    def test_get_openvas_result_hosts_count(self, MockDBClass):
+        w = DummyDaemon()
+        target_element = w.create_xml_target()
+        targets = OspRequest.process_target_element(target_element)
+        w.create_scan('123-456', targets, None, [])
+
+        results = [
+            "HOSTS_COUNT||| ||| ||| ||| |||4",
+        ]
+        MockDBClass.get_result.return_value = results
+        w.set_scan_total_hosts = MagicMock()
+
+        w.report_openvas_results(MockDBClass, '123-456')
+        w.set_scan_total_hosts.assert_called_with(
+            '123-456',
+            4,
+        )
+
+    @patch('ospd_openvas.daemon.BaseDB')
     @patch('ospd_openvas.daemon.ResultList.add_scan_alarm_to_list')
     def test_result_without_vt_oid(
         self, mock_add_scan_alarm_to_list, MockDBClass
@@ -822,11 +890,11 @@ class TestOspdOpenvas(TestCase):
         targets = OspRequest.process_target_element(target_element)
         w.create_scan('123-456', targets, None, [])
         w.scan_collection.scans_table['123-456']['results'] = list()
-        results = ["ALARM||| ||| ||| |||some alarm|||path", None]
+        results = ["ALARM||| ||| ||| ||| |||some alarm|||path", None]
         MockDBClass.get_result.return_value = results
         mock_add_scan_alarm_to_list.return_value = None
 
-        w.report_openvas_results(MockDBClass, '123-456', 'localhost')
+        w.report_openvas_results(MockDBClass, '123-456')
 
         assert_called_once(logging.Logger.warning)
 
@@ -839,30 +907,50 @@ class TestOspdOpenvas(TestCase):
 
         self.assertTrue(ret)
 
+    @patch('psutil.Process')
     @patch('ospd_openvas.db.KbDB')
-    def test_openvas_is_alive_still(self, mock_db):
+    def test_openvas_is_alive_still(self, mock_db, mock_psutil):
         w = DummyDaemon()
-        # mock_psutil = MockPsutil.return_value
+        mock_psutil.side_effect = TypeError
         mock_db.scan_is_stopped.return_value = False
-        ret = w.is_openvas_process_alive(mock_db, '1234', 'a1-b2-c3-d4')
+        ret = w.is_openvas_process_alive(mock_db, '1234', 'a1-b2-c3-d3')
 
         self.assertFalse(ret)
 
-    @patch('ospd_openvas.daemon.OSPDaemon.set_scan_host_progress')
-    def test_update_progress(self, mock_set_scan_host_progress):
+    @patch('ospd_openvas.daemon.OSPDaemon.set_scan_progress_batch')
+    @patch('ospd_openvas.daemon.OSPDaemon.sort_host_finished')
+    @patch('ospd_openvas.db.KbDB')
+    def test_report_openvas_scan_status(
+        self, mock_db, mock_sort_host_finished, mock_set_scan_progress_batch
+    ):
         w = DummyDaemon()
 
-        mock_set_scan_host_progress.return_value = None
+        mock_set_scan_progress_batch.return_value = None
+        mock_sort_host_finished.return_value = None
+        mock_db.get_scan_status.return_value = [
+            '192.168.0.1/15/1000',
+            '192.168.0.2/15/0',
+            '192.168.0.3/15/-1',
+            '192.168.0.4/1500/1500',
+        ]
 
-        msg = '0/-1'
         target_element = w.create_xml_target()
         targets = OspRequest.process_target_element(target_element)
 
         w.create_scan('123-456', targets, None, [])
-        w.update_progress('123-456', 'localhost', msg)
+        w.report_openvas_scan_status(mock_db, '123-456')
 
-        mock_set_scan_host_progress.assert_called_with(
-            '123-456', host='localhost', progress=-1
+        mock_set_scan_progress_batch.assert_called_with(
+            '123-456',
+            host_progress={
+                '192.168.0.1': 1,
+                '192.168.0.3': -1,
+                '192.168.0.4': 100,
+            },
+        )
+
+        mock_sort_host_finished.assert_called_with(
+            '123-456', ['192.168.0.3', '192.168.0.4']
         )
 
 
@@ -892,3 +980,26 @@ class TestFilters(TestCase):
             vts_collection, "modification_time>10"
         )
         self.assertIn('1.3.6.1.4.1.25623.1.0.100061', res)
+
+    def test_get_severity_score_v2(self):
+        w = DummyDaemon()
+        vtaux = {
+            'severities': {
+                'severity_type': 'cvss_base_v2',
+                'severity_base_vector': 'AV:N/AC:L/Au:N/C:P/I:N/A:N',
+            }
+        }
+
+        a = w.get_severity_score(vtaux)
+        self.assertEqual(w.get_severity_score(vtaux), 5.0)
+
+    def test_get_severity_score_v3(self):
+        w = DummyDaemon()
+        vtaux = {
+            'severities': {
+                'severity_type': 'cvss_base_v3',
+                'severity_base_vector': 'CVSS:3.0/AV:L/AC:H/PR:H/UI:R/S:U/C:N/I:L/A:L',
+            }
+        }
+
+        self.assertEqual(w.get_severity_score(vtaux), 2.9)
