@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (C) 2014-2020 Greenbone Networks GmbH
+# Copyright (C) 2014-2021 Greenbone Networks GmbH
 #
 # SPDX-License-Identifier: AGPL-3.0-or-later
 #
@@ -164,11 +164,23 @@ class TestOpenvasDB(TestCase):
         self.assertEqual(value, 'a')
         ctx.lindex.assert_called_once_with('a', 0)
 
+    def test_add_single_list(self, mock_redis):
+        ctx = mock_redis.return_value
+        pipeline = ctx.pipeline.return_value
+        pipeline.delete.return_value = None
+        pipeline.execute.return_value = (None, 0)
+
+        OpenvasDB.add_single_list(ctx, 'a', ['12', '11', '12'])
+
+        pipeline.delete.assert_called_once_with('a')
+        pipeline.rpush.assert_called_once_with('a', '12', '11', '12')
+        assert_called(pipeline.execute)
+
     def test_add_single_item(self, mock_redis):
         ctx = mock_redis.return_value
         ctx.rpush.return_value = 1
 
-        OpenvasDB.add_single_item(ctx, 'a', ['12'])
+        OpenvasDB.add_single_item(ctx, 'a', ['12', '12'])
 
         ctx.rpush.assert_called_once_with('a', '12')
 
@@ -196,6 +208,37 @@ class TestOpenvasDB(TestCase):
         with self.assertRaises(RequiredArgument):
             OpenvasDB.set_single_item(ctx, '1', None)
 
+    def test_pop_list_items_no_results(self, mock_redis):
+        ctx = mock_redis.return_value
+        pipeline = ctx.pipeline.return_value
+        pipeline.lrange.return_value = None
+        pipeline.delete.return_value = None
+        pipeline.execute.return_value = (None, 0)
+
+        ret = OpenvasDB.pop_list_items(ctx, 'foo')
+
+        self.assertEqual(ret, [])
+
+        pipeline.lrange.assert_called_once_with('foo', 0, -1)
+        pipeline.delete.assert_called_once_with('foo')
+        assert_called(pipeline.execute)
+
+    def test_pop_list_items_with_results(self, mock_redis):
+        ctx = mock_redis.return_value
+        pipeline = ctx.pipeline.return_value
+        pipeline.lrange.return_value = None
+        pipeline.delete.return_value = None
+        pipeline.execute.return_value = [['c', 'b', 'a'], 2]
+
+        ret = OpenvasDB.pop_list_items(ctx, 'results')
+
+        # reversed list
+        self.assertEqual(ret, ['a', 'b', 'c'])
+
+        pipeline.lrange.assert_called_once_with('results', 0, -1)
+        pipeline.delete.assert_called_once_with('results')
+        assert_called(pipeline.execute)
+
     def test_set_single_item(self, mock_redis):
         ctx = mock_redis.return_value
         pipeline = ctx.pipeline.return_value
@@ -319,7 +362,12 @@ class ScanDBTestCase(TestCase):
 
         ret = self.db.get_result()
 
-        self.assertEqual(ret, ['some result',])
+        self.assertEqual(
+            ret,
+            [
+                'some result',
+            ],
+        )
         mock_openvas_db.pop_list_items.assert_called_with(
             self.ctx, 'internal/results'
         )
@@ -342,76 +390,6 @@ class ScanDBTestCase(TestCase):
 
         mock_openvas_db.select_database.assert_called_with(self.ctx, 11)
 
-    def test_get_scan_id(self, mock_openvas_db):
-        mock_openvas_db.get_single_item.return_value = 'foo'
-
-        ret = self.db.get_scan_id()
-
-        self.assertEqual(ret, 'foo')
-        mock_openvas_db.get_single_item.assert_called_with(
-            self.ctx, 'internal/scan_id'
-        )
-
-    def test_get_scan_status(self, mock_openvas_db):
-        mock_openvas_db.get_last_list_item.return_value = 'foo'
-
-        ret = self.db.get_scan_status()
-
-        self.assertEqual(ret, 'foo')
-        mock_openvas_db.get_last_list_item.assert_called_with(
-            self.ctx, 'internal/status'
-        )
-
-    def test_get_host_scan_start_time(self, mock_openvas_db):
-        mock_openvas_db.get_last_list_item.return_value = 'some start time'
-
-        ret = self.db.get_host_scan_start_time()
-
-        self.assertEqual(ret, 'some start time')
-        mock_openvas_db.get_last_list_item.assert_called_with(
-            self.ctx, 'internal/start_time'
-        )
-
-    def test_get_host_scan_end_time(self, mock_openvas_db):
-        mock_openvas_db.get_last_list_item.return_value = 'some end time'
-
-        ret = self.db.get_host_scan_end_time()
-
-        self.assertEqual(ret, 'some end time')
-        mock_openvas_db.get_last_list_item.assert_called_with(
-            self.ctx, 'internal/end_time'
-        )
-
-    def test_get_host_ip(self, mock_openvas_db):
-        mock_openvas_db.get_single_item.return_value = '192.168.0.1'
-
-        ret = self.db.get_host_ip()
-
-        self.assertEqual(ret, '192.168.0.1')
-        mock_openvas_db.get_single_item.assert_called_with(
-            self.ctx, 'internal/ip'
-        )
-
-    def test_host_is_finished_false(self, mock_openvas_db):
-        mock_openvas_db.get_single_item.return_value = 'foo'
-
-        ret = self.db.host_is_finished('bar')
-
-        self.assertFalse(ret)
-        mock_openvas_db.get_single_item.assert_called_with(
-            self.ctx, 'internal/bar'
-        )
-
-    def test_host_is_finished_true(self, mock_openvas_db):
-        mock_openvas_db.get_single_item.return_value = 'finished'
-
-        ret = self.db.host_is_finished('bar')
-
-        self.assertTrue(ret)
-        mock_openvas_db.get_single_item.assert_called_with(
-            self.ctx, 'internal/bar'
-        )
-
     def test_flush(self, mock_openvas_db):
         self.db.flush()
 
@@ -432,7 +410,12 @@ class KbDBTestCase(TestCase):
 
         ret = self.db.get_result()
 
-        self.assertEqual(ret, ['some results',])
+        self.assertEqual(
+            ret,
+            [
+                'some results',
+            ],
+        )
         mock_openvas_db.pop_list_items.assert_called_with(
             self.ctx, 'internal/results'
         )
@@ -447,13 +430,28 @@ class KbDBTestCase(TestCase):
             self.ctx, 'internal/foo'
         )
 
+    def test_get_scan_status(self, mock_openvas_db):
+        status = [
+            '192.168.0.1/10/120',
+            '192.168.0.2/35/120',
+        ]
+
+        mock_openvas_db.pop_list_items.return_value = status
+
+        ret = self.db.get_scan_status()
+
+        self.assertEqual(ret, status)
+        mock_openvas_db.pop_list_items.assert_called_with(
+            self.ctx, 'internal/status'
+        )
+
     def test_flush(self, mock_openvas_db):
         self.db.flush()
 
         self.ctx.flushdb.assert_called_with()
 
     def test_add_scan_id(self, mock_openvas_db):
-        self.db.add_scan_id('foo', 'bar')
+        self.db.add_scan_id('bar')
 
         calls = mock_openvas_db.add_single_item.call_args_list
 
@@ -466,12 +464,6 @@ class KbDBTestCase(TestCase):
         call = calls[1]
         kwargs = call[0]
 
-        self.assertEqual(kwargs[1], 'internal/foo/globalscanid')
-        self.assertEqual(kwargs[2], ['bar'])
-
-        call = calls[2]
-        kwargs = call[0]
-
         self.assertEqual(kwargs[1], 'internal/scanid')
         self.assertEqual(kwargs[2], ['bar'])
 
@@ -531,9 +523,9 @@ class KbDBTestCase(TestCase):
         )
 
     def test_target_is_finished_false(self, mock_openvas_db):
-        mock_openvas_db.get_single_item.side_effect = ['bar', 'new']
+        mock_openvas_db.get_single_item.side_effect = ['new']
 
-        ret = self.db.target_is_finished('foo')
+        ret = self.db.target_is_finished('bar')
 
         self.assertFalse(ret)
 
@@ -542,17 +534,12 @@ class KbDBTestCase(TestCase):
         call = calls[0]
         args = call[0]
 
-        self.assertEqual(args[1], 'internal/foo/globalscanid')
-
-        call = calls[1]
-        args = call[0]
-
         self.assertEqual(args[1], 'internal/bar')
 
     def test_target_is_finished_true(self, mock_openvas_db):
-        mock_openvas_db.get_single_item.side_effect = ['bar', 'finished']
+        mock_openvas_db.get_single_item.side_effect = ['finished']
 
-        ret = self.db.target_is_finished('foo')
+        ret = self.db.target_is_finished('bar')
 
         self.assertTrue(ret)
 
@@ -561,19 +548,8 @@ class KbDBTestCase(TestCase):
         call = calls[0]
         args = call[0]
 
-        self.assertEqual(args[1], 'internal/foo/globalscanid')
-
-        call = calls[1]
-        args = call[0]
-
         self.assertEqual(args[1], 'internal/bar')
 
-        mock_openvas_db.get_single_item.side_effect = ['bar', None]
-
-        ret = self.db.target_is_finished('foo')
-
-        self.assertTrue(ret)
-
     def test_stop_scan(self, mock_openvas_db):
         self.db.stop_scan('foo')
 
@@ -746,35 +722,34 @@ class MainDBTestCase(TestCase):
 
         new_ctx = 'bar'  # just some object to compare
         mock_openvas_db.create_context.return_value = new_ctx
-        mock_openvas_db.get_single_item.return_value = None
+        mock_openvas_db.get_key_count.return_value = None
 
         maindb = MainDB(ctx)
         maindb._max_dbindex = 2  # pylint: disable=protected-access
 
-        scan_id, kbdb = maindb.find_kb_database_by_scan_id('foo')
+        kbdb = maindb.find_kb_database_by_scan_id('foo')
 
-        mock_openvas_db.get_single_item.assert_called_once_with(
-            new_ctx, 'internal/foo/globalscanid'
+        mock_openvas_db.get_key_count.assert_called_once_with(
+            new_ctx, 'internal/foo'
         )
-        self.assertIsNone(scan_id)
+
         self.assertIsNone(kbdb)
 
     @patch('ospd_openvas.db.OpenvasDB')
     def test_find_kb_database_by_scan_id(self, mock_openvas_db, mock_redis):
         ctx = mock_redis.return_value
 
-        new_ctx = 'bar'  # just some object to compare
+        new_ctx = 'foo'  # just some object to compare
         mock_openvas_db.create_context.return_value = new_ctx
-        mock_openvas_db.get_single_item.side_effect = [None, 'ipsum']
+        mock_openvas_db.get_key_count.side_effect = [0, 1]
 
         maindb = MainDB(ctx)
         maindb._max_dbindex = 3  # pylint: disable=protected-access
 
-        scan_id, kbdb = maindb.find_kb_database_by_scan_id('foo')
+        kbdb = maindb.find_kb_database_by_scan_id('foo')
 
-        mock_openvas_db.get_single_item.assert_called_with(
-            new_ctx, 'internal/foo/globalscanid'
+        mock_openvas_db.get_key_count.assert_called_with(
+            new_ctx, 'internal/foo'
         )
-        self.assertEqual(scan_id, 'ipsum')
         self.assertEqual(kbdb.index, 2)
         self.assertIs(kbdb.ctx, new_ctx)
